/*****************************************************************************
 * This file contains all the code needed for driving the PIC programmer. If *
 * you want to support other programmers, just modify this code and avoid    *
 * touch the other. And, please, send me the new file ;-)                    *
 *****************************************************************************/

#include <fcntl.h>
#include <sys/ioctl.h>

void publicidad(int);

int port_file;
int puerto;
char what_use=0;

/* This function initialices the ports, etc. The port number given in the
 * command line with the option -p is in the global variable PORTO. If the
 * port is not supported, it must return 1. If the IOPERM operation fails,
 * it must return a 2. If the file can't be opened, it must return 3.
 * If there's no error, it must return 0.
 * The variable WHAT_USE must be set to 0 if we use parallel port, or 1 if
 * we use serial port. */

char init() {
  int basura,dato;
  char the_file_to_open[100];

  if((porto>=0) && (porto<=2)) {
    what_use=0;
    if(porto==0)
      puerto=0x378;
    if(porto==1)
      puerto=0x278;
    if(porto==2)
      puerto=0x3bc;
    basura=ioperm(puerto,3,1);
    if(basura==-1)
      return(2);
  }
  if((porto>=3) && (porto<=6)) {
    what_use=1;
    if(porto==3)
      strcpy(the_file_to_open,"/dev/ttyS0");
    if(porto==4)
      strcpy(the_file_to_open,"/dev/ttyS1");
    if(porto==5)
      strcpy(the_file_to_open,"/dev/ttyS2");
    if(porto==6)
      strcpy(the_file_to_open,"/dev/ttyS3");
    port_file=open(the_file_to_open,O_RDWR);
    if(port_file==-1)
      return(3);

    dato=0; // current for the PIC
    ioctl(port_file,TIOCMSET,&dato);

    /* MCLR line -> 12 volts */

    dato=0;
    basura=ioctl(port_file,TIOCSBRK,&dato);
  }

  usleep(pausa2);

  if(porto>6)
    return(1);
  else
    return(0);
}

/* This function closes all the ports used. */

void ending() {
  int basura,dato;
  if(what_use==0)
    basura=ioperm(puerto,3,0);
  if(what_use==1) {
    dato=0;
    basura=ioctl(port_file,TIOCCBRK,&dato);
    close(port_file);
  }
}

/* This function resets the PIC. */

void reset() {

  int basura,dato;

  if(what_use==0) {
    outb(0,puerto); // MCLR to 12 volts (reseting the printer port)
    usleep(pausa2); // Wait PAUSA2 microseconds
    outb(4,puerto); // MCLR to 0 volts (RESET)
    usleep(pausa2); // Wait PAUSA2 microseconds
    outb(0,puerto); // MCLR to 12 volts (ready to be programmed)
    usleep(pausa2); // Wait PAUSA2 microseconds
  }
  if(what_use==1) {

    dato=0; // TXD -> 12 Volts
    basura=ioctl(port_file,TIOCSBRK,&dato);
    usleep(pausa2); // Wait PAUSA2 microseconds
    dato=0; // TXD -> 0 volts
    basura=ioctl(port_file,TIOCCBRK,&dato);
    usleep(pausa2); // Wait PAUSA2 microseconds
    dato=0; // TXD -> 12 volts
    basura=ioctl(port_file,TIOCSBRK,&dato);
    usleep(pausa2); //Wait PAUSA2 microseconds
  }
}

/* This function sends one bit to the PIC. */

void envia_bit(char bit) {
  int contador,dato,dato2,basura,basura2;

  if(what_use==0) {
    outb(bit+2,puerto); // rising edge
    for(contador=0;contador<pausa1;contador++)
      inb(puerto); // pause
    outb(bit,puerto); // falling edge
    for(contador=0;contador<pausa1;contador++)
      inb(puerto); // pause
  }
  if(what_use==1) {
    dato=(bit ? TIOCM_DTR : 0);
    dato2=dato|TIOCM_RTS;
    ioctl(port_file,TIOCMSET,&dato2); // rising edge
    for(contador=0;contador<pausa1;contador++)
      basura=ioctl(port_file,TIOCMGET,&basura2); // pause
    ioctl(port_file,TIOCMSET,&dato); // falling edge
    for(contador=0;contador<pausa1;contador++)
      basura=ioctl(port_file,TIOCMGET,&basura2); // pause    
  }
}

/* This function reads one bit from the PIC. */

char lee_bit(void) {
  char dato;
  int contador,data,data2,basura,basura2;

  if(what_use==0) {
    outb(1,puerto); // here we put the hardware in read mode (DATA bit set).
    outb(3,puerto); // rising edge
    
    for(contador=0;contador<pausa1;contador++)
      inb(puerto); // pause
    
    dato=inb(puerto+1)&32; // now, we read the bit in the right port
    if(dato!=0)
      dato=1;
    outb(1,puerto); // falling edge
    
    for(contador=0;contador<pausa1;contador++)
      inb(puerto); // pause
  }

  if(what_use==1) {
    data=TIOCM_DTR;
    data2=data | TIOCM_RTS;
    basura=ioctl(port_file,TIOCMSET,&data); // read mode (DTR bit set)
    basura=ioctl(port_file,TIOCMSET,&data2); // rising edge
    for(contador=0;contador<pausa1;contador++)
      basura=ioctl(port_file,TIOCMGET,&basura2); // pause
    basura=ioctl(port_file,TIOCMGET,&basura2); // we read the bit in basura
    if((basura2 & TIOCM_CTS)==0)
      dato=0;
    else
      dato=1;
    basura=ioctl(port_file,TIOCMSET,&data); // falling edge
    for(contador=0;contador<pausa1;contador++)
      basura=ioctl(port_file,TIOCMGET,&basura2); // pause
  }
  return (dato);
}


/* This function does a pause of TIEMPO microsecond, just reading a port */

void publicidad(int tiempo) {
  int basura,basura2,contador;
  char retorno;

  if(what_use==0)
    for(contador=0;contador<tiempo;contador++)
      retorno=inb(puerto);
  if(what_use==1) {
    if(tiempo>1)
      tiempo/=2;
      tiempo++;
    for(contador=0;contador<tiempo;contador++)
      basura=ioctl(port_file,TIOCMGET,&basura2);
  }
}
